@@ -38,7 +38,7 @@ At the moment, the parser builds only a parse tree, not an abstract syntax tree
 import os
 import sys
 import codecs
-from re import MULTILINE
+from re import MULTILINE, DOTALL
 from pprint import pformat
 from funcparserlib.util import pretty_tree
 from funcparserlib.lexer import make_tokenizer, Token, LexerError
@@ -56,6 +56,10 @@ Edge = namedtuple('Edge', 'nodes attrs')
 DefAttrs = namedtuple('DefAttrs', 'object attrs')
 
 
+class ParseException(Exception):
+    pass
+
+
 def tokenize(str):
     'str -> Sequence(Token)'
     specs = [
@@ -66,7 +70,7 @@ def tokenize(str):
         ('Name',    (ur'[A-Za-z_\u0080-\uffff][A-Za-z_0-9\u0080-\uffff]*',)),
         ('Op',      (r'[{};,=\[\]]|(<->)|(<-)|(--)|(->)',)),
         ('Number',  (r'-?(\.[0-9]+)|([0-9]+(\.[0-9]*)?)',)),
-        ('String',  (r'(?P<quote>"|\')(?:.|\s)*?(?<!\\)(?P=quote)',)),
+        ('String',  (r'(?P<quote>"|\').*?(?<!\\)(?P=quote)', DOTALL)),
     ]
     useless = ['Comment', 'NL', 'Space']
     t = make_tokenizer(specs)
@@ -190,24 +194,11 @@ def pretty_parse_tree(x):
 
 
 def parse_file(path):
-    input = codecs.open(path, 'r', 'utf-8').read()
-    return parse(tokenize(input))
-
-
-def main():
-    #import logging
-    #logging.basicConfig(level=logging.DEBUG)
-    #import funcparserlib
-    #funcparserlib.parser.debug = True
     try:
-        stdin = os.fdopen(sys.stdin.fileno(), 'rb')
-        input = stdin.read().decode(ENCODING)
-        tree = parse(tokenize(input))
-        print pretty_parse_tree(tree).encode(ENCODING)
-    except (NoParseError, LexerError), e:
-        msg = (u'syntax error: %s' % e).encode(ENCODING)
-        print >> sys.stderr, msg
-        sys.exit(1)
-
-if __name__ == '__main__':
-    main()
+        input = codecs.open(path, 'r', 'utf-8').read()
+        return parse(tokenize(input))
+    except LexerError, e:
+        message = "Got unexpected token at line %d column %d" % e.place
+        raise ParseException(message)
+    except Exception, e:
+        raise ParseException(str(e))
